@@ -50,7 +50,8 @@ static void meswin_row_activated_callback(GtkTreeView *view,
 					  GtkTreePath *path,
 					  GtkTreeViewColumn *col,
 					  gpointer data);
-static void meswin_response_callback(struct gui_dialog *dlg, int response);
+static void meswin_response_callback(struct gui_dialog *dlg, int response,
+                                     gpointer data);
 
 enum {
   CMD_GOTO = 1, CMD_POPCITY
@@ -59,9 +60,10 @@ enum {
 #define N_MSG_VIEW 24	       /* max before scrolling happens */
 
 /****************************************************************
-popup the dialog 10% inside the main-window 
+popup the dialog 10% inside the main-window, and optionally
+raise it.
 *****************************************************************/
-void popup_meswin_dialog(void)
+void popup_meswin_dialog(bool raise)
 {
   if (!meswin_shell) {
     create_meswin_dialog();
@@ -70,15 +72,9 @@ void popup_meswin_dialog(void)
   update_meswin_dialog();
 
   gui_dialog_present(meswin_shell);
-}
-
-/****************************************************************
- Raises the message window dialog.
-****************************************************************/
-void raise_meswin_dialog(void)
-{
-  popup_meswin_dialog();
-  gui_dialog_raise(meswin_shell);
+  if (raise) {
+    gui_dialog_raise(meswin_shell);
+  }
 }
 
 /**************************************************************************
@@ -102,27 +98,17 @@ bool is_meswin_open(void)
 /****************************************************************
 ...
 *****************************************************************/
-static void meswin_visited_item(gint n)
+static void meswin_set_visited(GtkTreeIter *it, bool visited)
 {
-  GtkTreeIter it;
+  GtkListStore *store;
+  gint row;
 
-  if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(meswin_store),&it,NULL,n)) {
-    gtk_list_store_set(meswin_store, &it, 1, (gint)TRUE, -1);
-    set_message_visited_state(n, TRUE);
-  }
-}
+  store = meswin_store;
+  g_return_if_fail(store != NULL);
 
-/****************************************************************
-...
-*****************************************************************/
-static void meswin_not_visited_item(gint n)
-{
-  GtkTreeIter it;
-
-  if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(meswin_store),&it,NULL,n)) {
-    gtk_list_store_set(meswin_store, &it, 1, (gint)FALSE, -1);
-    set_message_visited_state(n, FALSE);
-  }
+  gtk_list_store_set(store, it, 1, visited, -1);
+  gtk_tree_model_get(GTK_TREE_MODEL(store), it, 2, &row, -1);
+  set_message_visited_state(row, visited);
 }
 
 /****************************************************************
@@ -146,6 +132,42 @@ static void meswin_cell_data_func(GtkTreeViewColumn *col,
   }
 }
 					     
+/**************************************************************************
+  Mouse button press handler for the message window treeview. We only
+  care about right clicks on a row; this action centers on the tile
+  associated with the event at that row (if applicable).
+**************************************************************************/
+static gboolean meswin_button_press_callback(GtkWidget *widget,
+                                             GdkEventButton *ev,
+                                             gpointer data)
+{
+  GtkTreePath *path = NULL;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gint row;
+
+  g_return_val_if_fail(GTK_IS_TREE_VIEW(widget), FALSE);
+
+  if (ev->type != GDK_BUTTON_PRESS || ev->button != 3) {
+    return FALSE;
+  }
+
+  if (!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
+                                     (gint) ev->x, (gint) ev->y,
+                                     &path, NULL, NULL, NULL)) {
+    return TRUE;
+  }
+
+  model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+  if (gtk_tree_model_get_iter(model, &iter, path)) {
+    gtk_tree_model_get(model, &iter, 2, &row, -1);
+    meswin_goto(row);
+  }
+  gtk_tree_path_free(path);
+
+  return TRUE;
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -153,12 +175,19 @@ static void create_meswin_dialog(void)
 {
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *col;
-  GtkWidget *view, *sw, *cmd;
+  GtkWidget *view, *sw, *cmd, *notebook;
 
-  gui_dialog_new(&meswin_shell, GTK_NOTEBOOK(bottom_notebook));
+  if (split_bottom_notebook) {
+    notebook = right_notebook;
+  } else {
+    notebook = bottom_notebook;
+  }
+  gui_dialog_new(&meswin_shell, GTK_NOTEBOOK(notebook), NULL);
   gui_dialog_set_title(meswin_shell, _("Messages"));
 
-  meswin_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+  meswin_store = gtk_list_store_new(3, G_TYPE_STRING,
+                                    G_TYPE_BOOLEAN,
+                                    G_TYPE_INT);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
@@ -185,14 +214,18 @@ static void create_meswin_dialog(void)
 		   G_CALLBACK(meswin_selection_callback), NULL);
   g_signal_connect(view, "row_activated",
 		   G_CALLBACK(meswin_row_activated_callback), NULL);
+  g_signal_connect(view, "button-press-event",
+                   G_CALLBACK(meswin_button_press_callback), NULL);
 
-  cmd = gui_dialog_add_stockbutton(meswin_shell, GTK_STOCK_JUMP_TO,
-      _("Goto _location"), CMD_GOTO);
-  gtk_widget_set_sensitive(cmd, FALSE);
-  
-  cmd = gui_dialog_add_stockbutton(meswin_shell, GTK_STOCK_ZOOM_IN,
-      _("_Popup City"), CMD_POPCITY);
-  gtk_widget_set_sensitive(cmd, FALSE);
+  if (show_message_window_buttons) {
+    cmd = gui_dialog_add_stockbutton(meswin_shell, GTK_STOCK_JUMP_TO,
+                                     _("Goto _Location"), CMD_GOTO);
+    gtk_widget_set_sensitive(cmd, FALSE);
+
+    cmd = gui_dialog_add_stockbutton(meswin_shell, GTK_STOCK_ZOOM_IN,
+                                     _("Inspect _City"), CMD_POPCITY);
+    gtk_widget_set_sensitive(cmd, FALSE);
+  }
 
   gui_dialog_add_button(meswin_shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
@@ -200,8 +233,6 @@ static void create_meswin_dialog(void)
   gui_dialog_set_default_size(meswin_shell, 520, 300);
 
   gui_dialog_show_all(meswin_shell);
-
-  gtk_tree_view_focus(GTK_TREE_VIEW(view));
 }
 
 /**************************************************************************
@@ -209,25 +240,29 @@ static void create_meswin_dialog(void)
 **************************************************************************/
 void real_update_meswin_dialog(void)
 {
-  int i, num = get_num_messages(), num_not_visited = 0;
+  int i, num, num_not_visited = 0;
+  struct message *pmsg;
+  GtkListStore *store;
   GtkTreeIter it;
 
-  gtk_list_store_clear(meswin_store);
+  store = meswin_store;
+  g_return_if_fail(store != NULL);
+
+  gtk_list_store_clear(store);
+  num = get_num_messages();
 
   for (i = 0; i < num; i++) {
-    GValue value = { 0, };
+    pmsg = get_message(i);
 
-    gtk_list_store_append(meswin_store, &it);
-
-    g_value_init(&value, G_TYPE_STRING);
-    g_value_set_static_string(&value, get_message(i)->descr);
-    gtk_list_store_set_value(meswin_store, &it, 0, &value);
-    g_value_unset(&value);
-
-    if (get_message(i)->visited) {
-      meswin_visited_item(i);
+    if (new_messages_go_to_top) {
+      gtk_list_store_prepend(store, &it);
     } else {
-      meswin_not_visited_item(i);
+      gtk_list_store_append(store, &it);
+    }
+    gtk_list_store_set(store, &it, 0, pmsg->descr, 2, i, -1);
+    meswin_set_visited(&it, pmsg->visited);
+
+    if (!pmsg->visited) {
       num_not_visited++;
     }
   }
@@ -246,15 +281,23 @@ void real_update_meswin_dialog(void)
 static void meswin_selection_callback(GtkTreeSelection *selection,
 				      gpointer data)
 {
-  gint row = gtk_tree_selection_get_row(selection);
+  struct message *pmsg;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  gint row;
 
-  if (row != -1) {
-    struct message *message = get_message(row);
+  if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
+    return;
+  }
 
+  gtk_tree_model_get(model, &iter, 2, &row, -1);
+  pmsg = get_message(row);
+
+  if (pmsg) {
     gui_dialog_set_response_sensitive(meswin_shell, CMD_GOTO,
-	message->location_ok);
+                                      pmsg->location_ok);
     gui_dialog_set_response_sensitive(meswin_shell, CMD_POPCITY,
-	message->city_ok);
+                                      pmsg->city_ok);
   }
 }
 
@@ -266,50 +309,57 @@ static void meswin_row_activated_callback(GtkTreeView *view,
 					  GtkTreeViewColumn *col,
 					  gpointer data)
 {
-  gint row = gtk_tree_path_get_indices(path)[0];
-  struct message *message = get_message(row);
+  struct message *pmsg;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gint row;
+
+  model = gtk_tree_view_get_model(view);
+  if (!gtk_tree_model_get_iter(model, &iter, path)) {
+    return;
+  }
+
+  gtk_tree_model_get(model, &iter, 2, &row, -1);
+  pmsg = get_message(row);
 
   meswin_double_click(row);
-  meswin_visited_item(row);
+  meswin_set_visited(&iter, TRUE);
 
   gui_dialog_set_response_sensitive(meswin_shell, CMD_GOTO,
-      message->location_ok);
+                                    pmsg->location_ok);
   gui_dialog_set_response_sensitive(meswin_shell, CMD_POPCITY,
-      message->city_ok);
+                                    pmsg->city_ok);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void meswin_response_callback(struct gui_dialog *dlg, int response)
+static void meswin_response_callback(struct gui_dialog *dlg, int response,
+                                     gpointer data)
 {
-  switch (response) {
-  case CMD_GOTO:
-    {
-      gint row = gtk_tree_selection_get_row(meswin_selection);
+  GtkTreeSelection *sel;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gint row;
 
-      if (row == -1) {
-	return;
-      }
-
-      meswin_goto(row);
-      meswin_visited_item(row);
-    }
-    break;
-  case CMD_POPCITY:
-    {
-      gint row = gtk_tree_selection_get_row(meswin_selection);
-
-      if (row == -1) {
-	return;
-      }
-      meswin_popup_city(row);
-      meswin_visited_item(row);
-    }
-    break;
-  default:
+  if (response != CMD_GOTO && response != CMD_POPCITY) {
     gui_dialog_destroy(dlg);
-    break;
+    return;
   }
+
+  sel = meswin_selection;
+  g_return_if_fail(sel != NULL);
+  if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
+    return;
+  }
+
+  gtk_tree_model_get(model, &iter, 2, &row, -1);
+
+  if (response == CMD_GOTO) {
+    meswin_goto(row);
+  } else {
+    meswin_popup_city(row);
+  }
+  meswin_set_visited(&iter, TRUE);
 }
 
